from django.db import models
from django.db.models import Sum
from django.db.models.functions import Coalesce


def dettagli(casse, periodo):
    totale = 0
    imposta = 0
    imponibile = 0
    ndc = 0

    for cassa in casse:
        for chiusura in cassa.chiusure.filter(data__icontains=periodo):
            totale += chiusura.gran_totale
            imposta += chiusura.imposta
            imponibile += chiusura.imponibile
            ndc += chiusura.numero_documenti_commerciali

    return {
        "totale": totale,
        "imposta": imposta,
        "imponibile": imponibile,
        "ndc": ndc,
    }


def bilancio(azienda, periodo, negozio=None):
    # Entrate
    entrate_totale = 0
    entrate_imposta = 0
    entrate_imponibile = 0
    entrate_ndc = 0

    # Uscite
    uscite_totale = 0
    uscite_imposta = 0
    uscite_imponibile = 0

    casse = None

    if negozio:
        casse = negozio.casse.all()
    else:
        for negozio in azienda.negozi.all():
            casse = negozio.casse.all()

    # Chiusure Fiscali
    for cassa in casse:
        for chiusura in cassa.chiusure.all().filter(data__icontains=periodo):
            entrate_totale += chiusura.gran_totale
            entrate_imposta += chiusura.imposta
            entrate_imponibile += chiusura.imponibile
            entrate_ndc += chiusura.numero_documenti_commerciali

    # Fatture Vendita
    for fattura in azienda.fatture_vendita.all().filter(data_sdi__icontains=periodo):
        entrate_totale += fattura.totale
        entrate_imposta += fattura.imposta
        entrate_imponibile += fattura.imponibile
        entrate_ndc += 1

    # Fatture Acquisto
    for fattura in azienda.fatture_acquisto.all().filter(data_sdi__icontains=periodo):
        if fattura.tipo_documento.codice == "TD04":
            uscite_totale -= fattura.totale
            uscite_imposta -= fattura.imposta
            uscite_imponibile -= fattura.imponibile
        else:
            uscite_totale += fattura.totale
            uscite_imposta += fattura.imposta
            uscite_imponibile += fattura.imponibile

    context = {
        "periodo": periodo,
        "entrate": {
            "totale": entrate_totale,
            "imposta": entrate_imposta,
            "imponibile": entrate_imponibile,
            "ndc": entrate_ndc,
            "media_mensile": round(entrate_totale / 12, 2),
        },
        "uscite": {
            "totale": uscite_totale,
            "imposta": uscite_imposta,
            "imponibile": uscite_imponibile,
        },
        "saldo": {
            "totale": entrate_totale - uscite_totale,
            "imposta": entrate_imposta - uscite_imposta,
            "imponibile": entrate_imponibile - uscite_imponibile,
        },
    }

    # dettagli per mese
    if len(str(periodo)) == 4:
        context.update({"dettagli": {}})
        months = [
            "01",
            "02",
            "03",
            "04",
            "05",
            "06",
            "07",
            "08",
            "09",
            "10",
            "11",
            "12",
        ]

        for month in months:
            p = f"{periodo}-{month}"
            context["dettagli"].update({f"{month}": dettagli(casse, p)})

    return context
