from django.db.models import Sum
from django.db.models.functions import Coalesce

from minigest.docfisc.helpers.choices import XML7
from minigest.docfisc.models import ChiusuraFiscale, Fattura, FatturaRiepilogoIVA


def entrate(azienda, periodo, negozio=None):
    aliquote_iva = list()
    entrate = dict(
        {"periodo": periodo, "corrispettivi": {}, "saldo": {}, "ndc": 0, "ingressi": 0}
    )

    casse = None

    if negozio:
        casse = negozio.casse.all()
    else:
        for negozio in azienda.negozi.all():
            casse = negozio.casse.all()

        # Recupera le aliquote IVA dai corrispettivi
        for cassa in casse:
            for chiusura in cassa.chiusure.filter(data__icontains=periodo):
                for riepilogo in chiusura.reparti_iva.all():
                    if riepilogo.aliquota_iva not in aliquote_iva:
                        aliquote_iva.append(riepilogo.aliquota_iva)

    # Recupera le aliquote IVA dalle fatture
    for fattura in Fattura.objects.filter(
        fornitore=azienda, data_sdi__icontains=periodo
    ):
        for riepilogo in fattura.riepiloghi_iva.all():
            if riepilogo.aliquota_iva not in aliquote_iva:
                aliquote_iva.append(riepilogo.aliquota_iva)

    # Ordina le aliquote IVA
    aliquote_iva = sorted(aliquote_iva, key=lambda k: float(k))

    # Recupera il totale dei documenti commerciali
    tot_doc = ChiusuraFiscale.objects.filter(
        cassa__negozio__azienda=azienda, data__icontains=periodo
    ).aggregate(totale=Coalesce(Sum("numero_documenti_commerciali"), 0))

    # Recupera il totale degli ingressi
    tot_ingressi = ChiusuraFiscale.objects.filter(
        cassa__negozio__azienda=azienda, data__icontains=periodo
    ).aggregate(totale=Coalesce(Sum("ingressi"), 0))

    # Recupera il numero totale delle fatture
    tot_fat = Fattura.objects.filter(
        fornitore=azienda, data_sdi__icontains=periodo
    ).count()

    ## Aggiorna ndc e ingressi ##
    entrate["ndc"] = tot_doc.get("totale") + tot_fat
    entrate["ingressi"] = (
        tot_ingressi.get("totale")
        if tot_ingressi.get("totale") > entrate["ndc"]
        else entrate["ndc"]
    )

    ## CORRISPETTIVI ##
    for aliquota in aliquote_iva:
        entrate["corrispettivi"].update({str(aliquota): {}})

        totale = 0
        imposta = 0
        imponibile = 0

        for negozio in azienda.negozi.all():
            for cassa in negozio.casse.all():
                for chiusura in cassa.chiusure.filter(data__icontains=periodo):
                    for riepilogo in chiusura.reparti_iva.filter(aliquota_iva=aliquota):
                        totale += riepilogo.corrispettivo
                        imposta += riepilogo.imposta
                        imponibile += riepilogo.imponibile

                    for reso in chiusura.resi.filter(aliquota_iva=aliquota):
                        if chiusura.formato != XML7:
                            totale -= reso.corrispettivo
                            imposta -= reso.imposta
                            imponibile -= reso.imponibile

        for riepilogo in FatturaRiepilogoIVA.objects.filter(
            aliquota_iva=aliquota,
            fattura__fornitore=azienda,
            fattura__data__icontains=periodo,
        ):
            if riepilogo.fattura.tipo_documento.codice == "TD04":
                imposta -= riepilogo.imposta
                imponibile -= riepilogo.imponibile_importo
                totale -= riepilogo.fattura.importo_totale_documento
            else:
                imposta += riepilogo.imposta
                imponibile += riepilogo.imponibile_importo
                totale += riepilogo.fattura.importo_totale_documento

        entrate["corrispettivi"][str(aliquota)].update(
            {"totale": totale, "imposta": imposta, "imponibile": imponibile}
        )

    ## SALDO ##
    saldo_totale = 0
    saldo_imposta = 0
    saldo_imponibile = 0

    for negozio in azienda.negozi.all():
        for cassa in negozio.casse.all():
            for chiusura in cassa.chiusure.filter(data__icontains=periodo):
                saldo_totale += chiusura.gran_totale
                saldo_imposta += chiusura.imposta
                saldo_imponibile += chiusura.imponibile

    for riepilogo in FatturaRiepilogoIVA.objects.filter(
        fattura__fornitore=azienda,
        fattura__data__icontains=periodo,
    ):
        if riepilogo.fattura.tipo_documento.codice == "TD04":
            saldo_imposta -= riepilogo.imposta
            saldo_imponibile -= riepilogo.imponibile_importo
            saldo_totale -= riepilogo.fattura.importo_totale_documento
        else:
            saldo_imposta += riepilogo.imposta
            saldo_imponibile += riepilogo.imponibile_importo
            saldo_totale += riepilogo.fattura.importo_totale_documento

    entrate["saldo"].update(
        {
            "totale": saldo_totale,
            "imposta": saldo_imposta,
            "imponibile": saldo_imponibile,
        }
    )

    return entrate
