from datetime import datetime

from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponseNotFound
from django.shortcuts import render
from django.views import View

from minigest.anagrafica.models import SoggettoFiscale as Azienda
from minigest.pos.models import Negozio

from ..helpers import bilancio


class BilancioView(View):
    template_name = "contabilita/bilancio.html"

    def get(self, request, *args, **kwargs):
        azienda = None
        periodo = (
            datetime.now().strftime("%Y-%m")
            if "periodo" not in kwargs
            else kwargs["periodo"]
        )

        try:
            azienda = Azienda.objects.get(pk=kwargs["azienda"])
        except Azienda.DoesNotExist:
            return HttpResponseNotFound()

        pk_negozio = 0 if "negozio" not in request.GET else request.GET["negozio"]

        # Controlla che il negozio esista
        if pk_negozio == 0:
            negozio = None
        else:
            try:
                negozio = Negozio.objects.get(pk=pk_negozio)
                try:
                    azienda.negozi.get(pk=pk_negozio)
                except ObjectDoesNotExist:
                    return HttpResponseNotFound()
            except Negozio.DoesNotExist:
                return HttpResponseNotFound()

        context = {"azienda": azienda}
        context.update(bilancio(azienda, periodo, negozio))

        return render(request, self.template_name, context)
