from datetime import datetime

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from rest_framework import permissions, status
from rest_framework.response import Response
from rest_framework.views import APIView

from minigest.anagrafica.models import SoggettoFiscale as Azienda
from minigest.pos.models import Negozio

from ..helpers import bilancio

permission = permissions.AllowAny if settings.DEBUG else permissions.IsAuthenticated


class Bilancio(APIView):
    permission_classes = [permission]

    def get(self, request, pk, periodo=datetime.now().year):
        # inizializzazione delle variabili
        azienda = None

        # Controlla che l'azienda esista
        try:
            azienda = Azienda.objects.get(pk=pk)
        except Azienda.DoesNotExist:
            return Response("Azienda non selezionata", status=status.HTTP_404_NOT_FOUND)

        pk_negozio = 0 if "negozio" not in request.GET else request.GET["negozio"]

        # Controlla che il negozio esista
        if pk_negozio == 0:
            negozio = None
        else:
            try:
                negozio = Negozio.objects.get(pk=pk_negozio)
                try:
                    azienda.negozi.get(pk=pk_negozio)
                except ObjectDoesNotExist:
                    return Response(
                        "Il negozio non appartiene all'azienda",
                        status=status.HTTP_404_NOT_FOUND,
                    )
            except Negozio.DoesNotExist:
                return Response(
                    "Negozio non esistente", status=status.HTTP_404_NOT_FOUND
                )

        if len(str(periodo)) <= 4:
            periodo_precedente = int(periodo) - 1
        else:
            periodo = str(periodo).split("-")
            periodo_precedente = list()

            if len(periodo) == 1:
                periodo_precedente.append(str(int(periodo[0]) - 1))
            elif len(periodo) == 2:
                anno = str(int(periodo[0]) - 1)
                periodo_precedente = [anno, periodo[1]]
            else:
                anno = str(int(periodo[0]) - 1)
                periodo_precedente = [anno, periodo[1], periodo[2]]

            periodo = "-".join(periodo)
            periodo_precedente = "-".join(periodo_precedente)

        bilancio_uno = bilancio(azienda, periodo, negozio)
        bilancio_due = bilancio(azienda, periodo_precedente, negozio)

        context = [bilancio_uno, bilancio_due]

        return Response(context)
