from datetime import datetime

from django.http import HttpResponseNotFound
from django.shortcuts import render
from django.views import View

from minigest.anagrafica.models import SoggettoFiscale as Azienda
from minigest.docfisc.models import Fattura


class FattureAcquisto(View):
    template_name = "docfisc/fatture_acquisto.html"

    def get(self, request, *args, **kwargs):
        azienda = None
        periodo = (
            datetime.now().strftime("%Y-%m")
            if "periodo" not in kwargs
            else kwargs["periodo"]
        )

        try:
            azienda = Azienda.objects.get(pk=kwargs["azienda"])
        except Azienda.DoesNotExist:
            return HttpResponseNotFound()

        fatture = Fattura.objects.filter(
            cliente=azienda, data_sdi__icontains=periodo
        ).order_by("data_sdi")

        context = {"azienda": azienda, "periodo": periodo, "fatture": fatture}

        return render(request, self.template_name, context)
