from datetime import datetime

from django.conf import settings
from rest_framework import generics, permissions
from rest_framework.response import Response

from ..models import Fattura
from ..serializers import FatturaSerializer

permission = permissions.AllowAny if settings.DEBUG else permissions.IsAuthenticated


class FatturaAcquistoViewSet(generics.ListCreateAPIView):
    queryset = Fattura.objects.all()
    serializer_class = FatturaSerializer
    permission_classes = [permission]

    def list(self, request, *args, **kwargs):
        pk_azienda = kwargs["pk_azienda"]
        periodo = (
            datetime.today().strftime("%Y-%m")
            if "periodo" not in kwargs
            else kwargs["periodo"]
        )

        queryset = (
            self.get_queryset()
            .filter(cliente=pk_azienda, data_sdi__icontains=periodo)
            .order_by("data_sdi")
        )

        serializer = FatturaSerializer(queryset, many=True)
        return Response(serializer.data)
