from django.db import models
from django.db.models import Sum
from django.db.models.functions import Coalesce

from .carico_prodotto import CaricoMerceProdotto
from .inventario import Inventario
from .prodotto import Prodotto


class InventarioProdotto(models.Model):
    inventario = models.ForeignKey(
        Inventario, on_delete=models.CASCADE, related_name="prodotti"
    )

    prodotto = models.ForeignKey(Prodotto, on_delete=models.CASCADE)

    quantita = models.IntegerField(verbose_name="Quantità")

    @property
    def venduto(self):
        current = self.inventario
        prevs = Inventario.objects.filter(data__lt=self.inventario.data).order_by(
            "-data"
        )

        venduto = 0

        if len(prevs) == 1:
            prev = prevs
            prev = prev.first()
        elif len(prevs) > 1:
            prev = prevs[1]
        else:
            prev = None

        if prev is not None:
            totale_aquisti = CaricoMerceProdotto.objects.filter(
                carico__data__gte=prev.data,
                carico__data__lte=current.data,
                prodotto=self.prodotto,
            ).aggregate(totale=Coalesce(Sum("quantita"), 0))["totale"]

            ultima = prev.prodotti.get(prodotto=self.prodotto)
            giacenza_precedente = ultima.quantita

            venduto = (ultima.quantita + totale_aquisti) - self.quantita

        return venduto

    @property
    def totale_acquisti(self):
        current = self.inventario
        prevs = Inventario.objects.filter(data__lt=self.inventario.data).order_by(
            "-data"
        )

        totale = 0

        if len(prevs) == 1:
            prev = prevs
            prev = prev.first()
        elif len(prevs) > 1:
            prev = prevs[1]
        else:
            prev = None

        if prev is not None:
            totale = CaricoMerceProdotto.objects.filter(
                carico__data__gte=prev.data,
                carico__data__lte=current.data,
                prodotto=self.prodotto,
            ).aggregate(totale=Coalesce(Sum("quantita"), 0))["totale"]

        return totale

    @property
    def giacenza_precedente(self):
        prevs = Inventario.objects.filter(data__lt=self.inventario.data).order_by(
            "-data"
        )

        totale = 0

        if len(prevs) == 1:
            prev = prevs
            prev = prev.first()
        elif len(prevs) > 1:
            prev = prevs[1]
        else:
            prev = None

        if prev is not None:
            ultima = prev.prodotti.get(prodotto=self.prodotto)
            totale = ultima.quantita

        return totale

    def __str__(self):
        return f"{self.inventario.data} {self.quantita} {self.prodotto.nome}"

    class Meta:
        verbose_name = "Inventario Prodotto"
        verbose_name_plural = "Inventario Prodotti"
