import uuid

from django.db import models
from django.utils.text import slugify
from polymorphic.models import PolymorphicModel

from minigest.anagrafica.models import SoggettoFiscale

from .categoria import Categoria


class Prodotto(PolymorphicModel):
    uuid = models.UUIDField(default=uuid.uuid4, editable=False, unique=True)

    categoria = models.ForeignKey(
        Categoria,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        default=None,
    )

    fornitori = models.ManyToManyField(
        SoggettoFiscale, blank=True, default=None, related_name="prodotti"
    )

    mostra_sul_sito = models.BooleanField(default=False)

    nome = models.CharField(max_length=200)

    descrizione_breve = models.TextField(
        max_length=500, blank=True, null=True, default=None
    )
    descrizione = models.TextField(blank=True, null=True, default=None)

    url = models.CharField(max_length=500, blank=True, null=True, default=None)

    def save(self, *args, **kwargs):
        if self.url is None:
            if self.categoria is not None:
                nome = slugify(self.nome)
                url = f"{self.categoria.url}/{nome}"

                self.url = url
        super(Prodotto, self).save(*args, **kwargs)

    def __str__(self):
        return self.nome

    class Meta:
        verbose_name = "Prodotto"
        verbose_name_plural = "Prodotti"
        ordering = ("nome",)
