from django.db import models
from django.utils.text import slugify

from .prodotto import Prodotto
from .variante import Variante, VarianteValore


class ProdottoVariante(Prodotto):
    prodotto = models.ForeignKey(
        Prodotto, on_delete=models.CASCADE, related_name="varianti"
    )

    variante = models.ForeignKey(Variante, on_delete=models.CASCADE)
    variante_valore = models.ForeignKey(
        VarianteValore,
        on_delete=models.CASCADE,
    )

    descrizione = None

    def save(self, *args, **kwargs):
        self.categoria = self.prodotto.categoria
        self.nome = f"{self.prodotto.nome} {self.variante.variante} {self.variante_valore.valore}"

        # url
        variante = slugify(self.variante.variante)
        valore = slugify(self.variante_valore.valore)
        self.url = f"{self.prodotto.url}/{variante}-{valore}"
        super(ProdottoVariante, self).save(*args, **kwargs)

    def __str__(self):
        return f"{self.prodotto.nome} {self.variante.variante} {self.variante_valore.valore}"

    class Meta:
        verbose_name = "Prodotto Variante"
        verbose_name_plural = "Prodotto Varianti"
