from rest_framework import serializers

from ..models import Inventario, InventarioProdotto


class InventarioProdottoSerializer(serializers.ModelSerializer):
    prodotto_nome = serializers.CharField(source="prodotto.nome", read_only=True)

    class Meta:
        model = InventarioProdotto
        fields = [
            "id",
            "prodotto",
            "prodotto_nome",
            "quantita",
            "prodotto",
            "totale_acquisti",
            "venduto",
            "giacenza_precedente",
        ]
        read_only_fields = ["prodotto_nome", "venduto"]


class InventarioSerializer(serializers.ModelSerializer):
    prodotti = InventarioProdottoSerializer(many=True)

    class Meta:
        model = Inventario
        fields = ["id", "data", "prodotti"]

    def create(self, validated_data):
        inventario_ps = validated_data.pop("prodotti")

        inventario = Inventario.objects.create(**validated_data)
        for prodotto in inventario_ps:
            InventarioProdotto.objects.create(inventario=inventario, **prodotto)
        return inventario
