import datetime

from django.test import TestCase

from ..models import (
    CaricoMerce,
    CaricoMerceProdotto,
    Categoria,
    Inventario,
    InventarioProdotto,
    Prodotto,
)


class InventarioTestCase(TestCase):
    def setUp(self):
        # Crea la categoria
        categoria = Categoria.objects.create(
            nome="Duplicazione Chiavi",
            descrizione="Duplichiamo chiavi domestiche anche di sicurezza",
        )

        # Crea gli articoli
        UN5D = Prodotto.objects.create(categoria=categoria, nome="UN5D")

        # Crea un Inventario
        inventario_1 = Inventario.objects.create()
        inventario_1.data = datetime.date(2021, 12, 6)
        inventario_1.save()

        inventario_2 = Inventario.objects.create()

        # Aggiunge articoli all'inventario 1
        InventarioProdotto.objects.create(
            inventario=inventario_1, prodotto=UN5D, quantita=30
        )

        # Aggiunge articoli all'inventario 2
        InventarioProdotto.objects.create(
            inventario=inventario_2, prodotto=UN5D, quantita=40
        )

        # Carica la merce
        caricomerce_1 = CaricoMerce.objects.create(data=datetime.date(2021, 12, 10))
        caricomerce_2 = CaricoMerce.objects.create(data=datetime.date(2021, 12, 20))

        # Carica la merce 1
        CaricoMerceProdotto.objects.create(
            carico=caricomerce_1, prodotto=UN5D, quantita=100
        )

        # Carica la merce 2
        CaricoMerceProdotto.objects.create(
            carico=caricomerce_2, prodotto=UN5D, quantita=200
        )

    def test_venduto(self):
        inventario = Inventario.objects.latest("data")
        prodotto = Prodotto.objects.get(nome="UN5D")
        UN5D = InventarioProdotto.objects.get(inventario=inventario, prodotto=prodotto)

        self.assertEqual(UN5D.venduto, 290)
        self.assertEqual(UN5D.totale_acquisti, 300)
        self.assertEqual(UN5D.giacenza_precedente, 30)
