from operator import itemgetter

from django.conf import settings
from rest_framework import permissions
from rest_framework.response import Response
from rest_framework.views import APIView

from minigest.magazzino.models import Inventario, Prodotto

permission = permissions.AllowAny if settings.DEBUG else permissions.IsAuthenticated


class BilancioProdotti(APIView):
    permission_classes = [permission]

    def get(self, request):
        context = list()

        if "search" in self.request.GET:
            if self.request.GET["search"] is not None:
                search = self.request.GET["search"]
            else:
                search = ""

            prodotti = Prodotto.objects.filter(nome__icontains=search).order_by("nome")
        else:
            prodotti = Prodotto.objects.all().order_by("nome")

        for prodotto in prodotti:
            subcontext = {
                "inventario": dict(),
                "nome": prodotto.nome,
                "quantita": 0,
                "totale_acquisti": 0,
                "venduto": 0,
            }

            try:
                p = Inventario.objects.latest("data").prodotti.get(prodotto=prodotto)
                subcontext.update(
                    {
                        "inventario": {
                            "id": p.inventario.id,
                            "data": p.inventario.data,
                        },
                        "quantita": p.quantita,
                        "totale_acquisti": p.totale_acquisti,
                        "venduto": p.venduto,
                    }
                )
            except:
                pass

            context.append(subcontext)

        if "order_by" in self.request.GET:
            sort_by = (
                self.request.GET["order_by"]
                if self.request.GET["order_by"] is not None
                else "venduto"
            )
            sorted_context = sorted(context, key=lambda d: d[sort_by], reverse=True)
        else:
            sorted_context = sorted(context, key=lambda d: d["venduto"], reverse=True)

        return Response(sorted_context)
