from django.conf import settings
from django.db.models import Sum
from django.db.models.functions import Coalesce
from rest_framework import permissions, status
from rest_framework.response import Response
from rest_framework.views import APIView

from ..models import CaricoMerceProdotto, Inventario

permission = permissions.AllowAny if settings.DEBUG else permissions.IsAuthenticated


class InventarioView(APIView):
    permission_classes = [permission]

    def get(self, request, pk):
        current = None
        prev = None
        context = {"data": None, "prodotti": list()}

        try:
            current = Inventario.objects.get(pk=pk)
            context.update({"data": current.data})

        except Inventario.DoesNotExist:
            return Response(
                "Inventario corrente inesistente", status=status.HTTP_404_NOT_FOUND
            )

        try:
            prev = Inventario.objects.get(pk=pk - 1)
        except Inventario.DoesNotExist:
            pass

        for p in current.prodotti.all():
            sub_context = {
                "prodotto": p.prodotto.nome,
                "totale_acquisti": 0,
                "giacenza_precedente": 0,
                "giacenza_attuale": p.quantita,
                "venduto": 0,
            }

            if prev is not None:
                sub_context["totale_acquisti"] = CaricoMerceProdotto.objects.filter(
                    carico__data__gte=prev.data,
                    carico__data__lte=current.data,
                    prodotto=p.prodotto,
                ).aggregate(totale=Coalesce(Sum("quantita"), 0))["totale"]

                # Giacenza Precedente
                ultima = prev.prodotti.get(prodotto=p.prodotto)
                sub_context["giacenza_precedente"] = ultima.quantita
                sub_context["venduto"] = p.venduto

            context["prodotti"].append(sub_context)

        return Response(context)
